@@ -49,6 +49,18 @@ class TOSCAException(Exception):
     def __str__(self):
         return self.message
 
+    @staticmethod
+    def generate_inv_schema_property_error(self, attr, value, valid_values):
+        msg = (_('Schema definition of "%(propname)s" has '
+                 '"%(attr)s" attribute with invalid value '
+                 '"%(value1)s". The value must be one of '
+                 '"%(value2)s".') % {"propname": self.name,
+                                     "attr": attr,
+                                     "value1": value,
+                                     "value2": valid_values})
+        ExceptionCollector.appendException(
+            InvalidSchemaError(message=msg))
+
     @staticmethod
     def set_fatal_format_exception(flag):
         if isinstance(flag, bool):
@@ -76,6 +88,15 @@ class InvalidTypeError(TOSCAException):
     msg_fmt = _('Type "%(what)s" is not a valid type.')
 
 
+class InvalidTypeAdditionalRequirementsError(TOSCAException):
+    msg_fmt = _('Additional requirements for type "%(type)s" not met.')
+
+
+class RangeValueError(TOSCAException):
+    msg_fmt = _('The value "%(pvalue)s" of property "%(pname)s" is out of '
+                'range "(min:%(vmin)s, max:%(vmax)s)".')
+
+
 class InvalidSchemaError(TOSCAException):
     msg_fmt = _('%(message)s')
 
@@ -105,6 +126,22 @@ class URLException(TOSCAException):
     msg_fmt = _('%(what)s')
 
 
+class ToscaExtImportError(TOSCAException):
+    msg_fmt = _('Unable to import extension "%(ext_name)s". '
+                'Check to see that it exists and has no '
+                'language definition errors.')
+
+
+class ToscaExtAttributeError(TOSCAException):
+    msg_fmt = _('Missing attribute in extension "%(ext_name)s". '
+                'Check to see that it has required attributes '
+                '"%(attrs)s" defined.')
+
+
+class InvalidGroupTargetException(TOSCAException):
+    msg_fmt = _('"%(message)s"')
+
+
 class ExceptionCollector(object):
 
     exceptions = []
