@@ -16,6 +16,7 @@ from toscaparser.elements.statefulentitytype import StatefulEntityType
 
 class CapabilityTypeDef(StatefulEntityType):
     '''TOSCA built-in capabilities type.'''
+    TOSCA_TYPEURI_CAPABILITY_ROOT = 'tosca.capabilities.Root'
 
     def __init__(self, name, ctype, ntype, custom_def=None):
         self.name = name
@@ -25,7 +26,7 @@ class CapabilityTypeDef(StatefulEntityType):
         self.properties = None
         if self.PROPERTIES in self.defs:
             self.properties = self.defs[self.PROPERTIES]
-        self.parent_capabilities = self._get_parent_capabilities()
+        self.parent_capabilities = self._get_parent_capabilities(custom_def)
 
     def get_properties_def_objects(self):
         '''Return a list of property definition objects.'''
@@ -57,12 +58,15 @@ class CapabilityTypeDef(StatefulEntityType):
         if props_def and name in props_def:
             return props_def[name].value
 
-    def _get_parent_capabilities(self):
+    def _get_parent_capabilities(self, custom_def=None):
         capabilities = {}
         parent_cap = self.parent_type
         if parent_cap:
-            while parent_cap != 'tosca.capabilities.Root':
-                capabilities[parent_cap] = self.TOSCA_DEF[parent_cap]
+            while parent_cap != self.TOSCA_TYPEURI_CAPABILITY_ROOT:
+                if parent_cap in self.TOSCA_DEF.keys():
+                    capabilities[parent_cap] = self.TOSCA_DEF[parent_cap]
+                elif custom_def and parent_cap in custom_def.keys():
+                    capabilities[parent_cap] = custom_def[parent_cap]
                 parent_cap = capabilities[parent_cap]['derived_from']
         return capabilities
 
