@@ -18,6 +18,7 @@ import toscaparser
 from toscaparser.common.exception import ExceptionCollector
 from toscaparser.common.exception import InvalidSchemaError
 from toscaparser.common.exception import ValidationError
+from toscaparser.elements.portspectype import PortSpec
 from toscaparser.elements import scalarunit
 from toscaparser.utils.gettextutils import _
 
@@ -33,15 +34,15 @@ class Schema(collections.Mapping):
     )
 
     PROPERTY_TYPES = (
-        INTEGER, STRING, BOOLEAN, FLOAT,
+        INTEGER, STRING, BOOLEAN, FLOAT, RANGE,
         NUMBER, TIMESTAMP, LIST, MAP,
         SCALAR_UNIT_SIZE, SCALAR_UNIT_FREQUENCY, SCALAR_UNIT_TIME,
-        PORTDEF, VERSION
+        VERSION, PORTDEF, PORTSPEC
     ) = (
-        'integer', 'string', 'boolean', 'float',
+        'integer', 'string', 'boolean', 'float', 'range',
         'number', 'timestamp', 'list', 'map',
         'scalar-unit.size', 'scalar-unit.frequency', 'scalar-unit.time',
-        'PortDef', 'version'
+        'version', 'PortDef', PortSpec.SHORTNAME
     )
 
     SCALAR_UNIT_SIZE_DEFAULT = 'B'
@@ -368,15 +369,16 @@ class InRange(Constraint):
     Constrains a property or parameter to a value in range of (inclusive)
     the two values declared.
     """
+    UNBOUNDED = 'UNBOUNDED'
 
     constraint_key = Constraint.IN_RANGE
 
     valid_types = (int, float, datetime.date,
-                   datetime.time, datetime.datetime)
+                   datetime.time, datetime.datetime, str)
 
     valid_prop_types = (Schema.INTEGER, Schema.FLOAT, Schema.TIMESTAMP,
                         Schema.SCALAR_UNIT_SIZE, Schema.SCALAR_UNIT_FREQUENCY,
-                        Schema.SCALAR_UNIT_TIME)
+                        Schema.SCALAR_UNIT_TIME, Schema.RANGE)
 
     def __init__(self, property_name, property_type, constraint):
         super(InRange, self).__init__(property_name, property_type, constraint)
@@ -386,21 +388,31 @@ class InRange(Constraint):
                 InvalidSchemaError(message=_('The property "in_range" '
                                              'expects a list.')))
 
+        msg = _('The property "in_range" expects comparable values.')
         for value in self.constraint_value:
             if not isinstance(value, self.valid_types):
                 ExceptionCollector.appendException(
-                    InvalidSchemaError(_('The property "in_range" expects '
-                                         'comparable values.')))
+                    InvalidSchemaError(message=msg))
+            # The only string we allow for range is the special value
+            # 'UNBOUNDED'
+            if(isinstance(value, str) and value != self.UNBOUNDED):
+                ExceptionCollector.appendException(
+                    InvalidSchemaError(message=msg))
 
         self.min = self.constraint_value[0]
         self.max = self.constraint_value[1]
 
     def _is_valid(self, value):
-        if value < self.min:
+        if not isinstance(self.min, str):
+            if value < self.min:
+                return False
+        elif self.min != self.UNBOUNDED:
             return False
-        if value > self.max:
+        if not isinstance(self.max, str):
+            if value > self.max:
+                return False
+        elif self.max != self.UNBOUNDED:
             return False
-
         return True
 
     def _err_msg(self, value):
@@ -487,7 +499,7 @@ class MinLength(Constraint):
 
     valid_types = (int, )
 
-    valid_prop_types = (Schema.STRING, )
+    valid_prop_types = (Schema.STRING, Schema.MAP)
 
     def __init__(self, property_name, property_type, constraint):
         super(MinLength, self).__init__(property_name, property_type,
@@ -498,7 +510,8 @@ class MinLength(Constraint):
                                              'expects an integer.')))
 
     def _is_valid(self, value):
-        if isinstance(value, str) and len(value) >= self.constraint_value:
+        if ((isinstance(value, str) or isinstance(value, dict)) and
+           len(value) >= self.constraint_value):
             return True
 
         return False
@@ -521,7 +534,7 @@ class MaxLength(Constraint):
 
     valid_types = (int, )
 
-    valid_prop_types = (Schema.STRING, )
+    valid_prop_types = (Schema.STRING, Schema.MAP)
 
     def __init__(self, property_name, property_type, constraint):
         super(MaxLength, self).__init__(property_name, property_type,
@@ -532,7 +545,8 @@ class MaxLength(Constraint):
                                              'expects an integer.')))
 
     def _is_valid(self, value):
-        if isinstance(value, str) and len(value) <= self.constraint_value:
+        if ((isinstance(value, str) or isinstance(value, dict)) and
+           len(value) <= self.constraint_value):
             return True
 
         return False
