@@ -18,15 +18,20 @@ import six
 from toscaparser.common.exception import ExceptionCollector
 from toscaparser.common.exception import UnknownInputError
 from toscaparser.dataentity import DataEntity
+from toscaparser.elements.entity_type import EntityType
+from toscaparser.elements.relationshiptype import RelationshipType
 from toscaparser.utils.gettextutils import _
 
 
 GET_PROPERTY = 'get_property'
 GET_ATTRIBUTE = 'get_attribute'
 GET_INPUT = 'get_input'
+CONCAT = 'concat'
 
 SELF = 'SELF'
 HOST = 'HOST'
+TARGET = 'TARGET'
+SOURCE = 'SOURCE'
 
 HOSTED_ON = 'tosca.relationships.HostedOn'
 
@@ -119,18 +124,34 @@ class GetAttribute(Function):
 
     * { get_attribute: [ server, private_address ] }
     * { get_attribute: [ HOST, private_address ] }
+    * { get_attribute: [ HOST, private_address, 0 ] }
     """
 
     def validate(self):
-        if len(self.args) != 2:
+        if len(self.args) != 2 and len(self.args) != 3:
             ExceptionCollector.appendException(
                 ValueError(_('Illegal arguments for function "{0}". Expected '
                              'arguments: "node-template-name", '
                              '"attribute-name"').format(GET_ATTRIBUTE)))
-        self._find_node_template_containing_attribute()
+        node_tpl = self._find_node_template_containing_attribute()
+        if len(self.args) > 2:
+            # Currently we only check the first level
+            attrs_def = node_tpl.type_definition.get_attributes_def()
+            attr_def = attrs_def[self.attribute_name]
+            if attr_def.schema['type'] == "list":
+                if not isinstance(self.args[2], int):
+                    ExceptionCollector.appendException(
+                        ValueError(_('Illegal arguments for function "{0}". '
+                                     'Third argument must be a positive'
+                                     ' integer') .format(GET_ATTRIBUTE)))
+            elif attr_def.schema['type'] != "map":
+                ExceptionCollector.appendException(
+                    ValueError(_('Illegal arguments for function "{0}". '
+                                 'Expected arguments: "node-template-name", '
+                                 '"attribute-name"').format(GET_ATTRIBUTE)))
 
     def result(self):
-        return self.args
+        return self
 
     def get_referenced_node_template(self):
         """Gets the NodeTemplate instance the get_attribute function refers to.
@@ -179,7 +200,6 @@ class GetAttribute(Function):
     def _find_host_containing_attribute(self, node_template_name=SELF):
         node_template = self._find_node_template(node_template_name)
         if node_template:
-            from toscaparser.elements.entity_type import EntityType
             hosted_on_rel = EntityType.TOSCA_DEF[HOSTED_ON]
             for r in node_template.requirements:
                 for requirement, target_name in r.items():
@@ -194,6 +214,20 @@ class GetAttribute(Function):
                                 target_name)
 
     def _find_node_template(self, node_template_name):
+        if node_template_name == TARGET:
+            if not isinstance(self.context.type_definition, RelationshipType):
+                ExceptionCollector.appendException(
+                    KeyError(_('"TARGET" keyword can only be used in context'
+                               ' to "Relationships" target node')))
+                return
+            return self.context.target
+        if node_template_name == SOURCE:
+            if not isinstance(self.context.type_definition, RelationshipType):
+                ExceptionCollector.appendException(
+                    KeyError(_('"SOURCE" keyword can only be used in context'
+                               ' to "Relationships" source node')))
+                return
+            return self.context.source
         name = self.context.name \
             if node_template_name == SELF and \
             not isinstance(self.context, list) \
@@ -220,7 +254,7 @@ class GetProperty(Function):
 
     Arguments:
 
-    * Node template name.
+    * Node template name | SELF | HOST | SOURCE | TARGET.
     * Requirement or capability name (optional).
     * Property name.
 
@@ -239,10 +273,11 @@ class GetProperty(Function):
     * { get_property: [ mysql_server, port ] }
     * { get_property: [ SELF, db_port ] }
     * { get_property: [ SELF, database_endpoint, port ] }
+    * { get_property: [ SELF, database_endpoint, port, 1 ] }
     """
 
     def validate(self):
-        if len(self.args) < 2 or len(self.args) > 3:
+        if len(self.args) < 2:
             ExceptionCollector.appendException(
                 ValueError(_(
                     'Expected arguments: "node-template-name", "req-or-cap" '
@@ -255,15 +290,31 @@ class GetProperty(Function):
             prop = found_prop.value
             if not isinstance(prop, Function):
                 get_function(self.tosca_tpl, self.context, prop)
-        elif len(self.args) == 3:
-            get_function(self.tosca_tpl,
-                         self.context,
-                         self._find_req_or_cap_property(self.args[1],
-                                                        self.args[2]))
-        else:
-            ExceptionCollector.appendException(
-                NotImplementedError(_(
-                    'Nested properties are not supported.')))
+        elif len(self.args) >= 3:
+            # do not use _find_property to avoid raise KeyError
+            # if the prop is not found
+            # First check if there is property with this name
+            node_tpl = self._find_node_template(self.args[0])
+            props = node_tpl.get_properties() if node_tpl else []
+            index = 2
+            found = [props[self.args[1]]] if self.args[1] in props else []
+            if found:
+                property_value = found[0].value
+            else:
+                index = 3
+                # then check the req or caps
+                property_value = self._find_req_or_cap_property(self.args[1],
+                                                                self.args[2])
+            if len(self.args) > index:
+                for elem in self.args[index:]:
+                    if isinstance(property_value, list):
+                        int_elem = int(elem)
+                        property_value = self._get_index_value(property_value,
+                                                               int_elem)
+                    else:
+                        property_value = self._get_attribute_value(
+                            property_value,
+                            elem)
 
     def _find_req_or_cap_property(self, req_or_cap, property_name):
         node_tpl = self._find_node_template(self.args[0])
@@ -328,6 +379,23 @@ class GetProperty(Function):
     def _find_node_template(self, node_template_name):
         if node_template_name == SELF:
             return self.context
+        # enable the HOST value in the function
+        if node_template_name == HOST:
+            return self._find_host_containing_property()
+        if node_template_name == TARGET:
+            if not isinstance(self.context.type_definition, RelationshipType):
+                ExceptionCollector.appendException(
+                    KeyError(_('"TARGET" keyword can only be used in context'
+                               ' to "Relationships" target node')))
+                return
+            return self.context.target
+        if node_template_name == SOURCE:
+            if not isinstance(self.context.type_definition, RelationshipType):
+                ExceptionCollector.appendException(
+                    KeyError(_('"SOURCE" keyword can only be used in context'
+                               ' to "Relationships" source node')))
+                return
+            return self.context.source
         if not hasattr(self.tosca_tpl, 'nodetemplates'):
             return
         for node_template in self.tosca_tpl.nodetemplates:
@@ -338,10 +406,98 @@ class GetProperty(Function):
                 'Node template "{0}" was not found.'
                 ).format(node_template_name)))
 
+    def _get_index_value(self, value, index):
+        if isinstance(value, list):
+            if index < len(value):
+                return value[index]
+            else:
+                ExceptionCollector.appendException(
+                    KeyError(_(
+                        "Property '{0}' found in capability '{1}'"
+                        " referenced from node template {2}"
+                        " must have an element with index {3}.").
+                        format(self.args[2],
+                               self.args[1],
+                               self.context.name,
+                               index)))
+        else:
+            ExceptionCollector.appendException(
+                KeyError(_(
+                    "Property '{0}' found in capability '{1}'"
+                    " referenced from node template {2}"
+                    " must be a list.").format(self.args[2],
+                                               self.args[1],
+                                               self.context.name)))
+
+    def _get_attribute_value(self, value, attibute):
+        if isinstance(value, dict):
+            if attibute in value:
+                return value[attibute]
+            else:
+                ExceptionCollector.appendException(
+                    KeyError(_(
+                        "Property '{0}' found in capability '{1}'"
+                        " referenced from node template {2}"
+                        " must have an attribute named {3}.").
+                        format(self.args[2],
+                               self.args[1],
+                               self.context.name,
+                               attibute)))
+        else:
+            ExceptionCollector.appendException(
+                KeyError(_(
+                    "Property '{0}' found in capability '{1}'"
+                    " referenced from node template {2}"
+                    " must be a dict.").format(self.args[2],
+                                               self.args[1],
+                                               self.context.name)))
+
+    # Add this functions similar to get_attribute case
+    def _find_host_containing_property(self, node_template_name=SELF):
+        node_template = self._find_node_template(node_template_name)
+        hosted_on_rel = EntityType.TOSCA_DEF[HOSTED_ON]
+        for r in node_template.requirements:
+            for requirement, target_name in r.items():
+                target_node = self._find_node_template(target_name)
+                target_type = target_node.type_definition
+                for capability in target_type.get_capabilities_objects():
+                    if capability.type in hosted_on_rel['valid_target_types']:
+                        if self._property_exists_in_type(target_type):
+                            return target_node
+                        return self._find_host_containing_property(
+                            target_name)
+        return None
+
+    def _property_exists_in_type(self, type_definition):
+        props_def = type_definition.get_properties_def()
+        found = [props_def[self.args[1]]] \
+            if self.args[1] in props_def else []
+        return len(found) == 1
+
     def result(self):
-        if len(self.args) == 3:
-            property_value = self._find_req_or_cap_property(self.args[1],
-                                                            self.args[2])
+        if len(self.args) >= 3:
+            # First check if there is property with this name
+            node_tpl = self._find_node_template(self.args[0])
+            props = node_tpl.get_properties() if node_tpl else []
+            index = 2
+            found = [props[self.args[1]]] if self.args[1] in props else []
+            if found:
+                property_value = found[0].value
+            else:
+                index = 3
+                # then check the req or caps
+                property_value = self._find_req_or_cap_property(self.args[1],
+                                                                self.args[2])
+            if len(self.args) > index:
+                for elem in self.args[index:]:
+                    if isinstance(property_value, list):
+                        int_elem = int(elem)
+                        property_value = self._get_index_value(property_value,
+                                                               int_elem)
+                    else:
+                        property_value = self._get_attribute_value(
+                            property_value,
+                            elem)
         else:
             property_value = self._find_property(self.args[1]).value
         if isinstance(property_value, Function):
@@ -367,10 +523,39 @@ class GetProperty(Function):
         return None
 
 
+class Concat(Function):
+    """Validate the function and provide an instance of the function
+
+    Concatenation of values are supposed to be produced at runtime and
+    therefore its the responsibility of the TOSCA engine to implement the
+    evaluation of Concat functions.
+
+    Arguments:
+
+    * List of strings that needs to be concatenated
+
+    Example:
+
+      [ 'http://',
+        get_attribute: [ server, public_address ],
+        ':' ,
+        get_attribute: [ server, port ] ]
+    """
+
+    def validate(self):
+        if len(self.args) < 1:
+            ExceptionCollector.appendException(
+                ValueError(_('Invalid arguments for function "{0}". Expected '
+                             'at least one arguments.').format(CONCAT)))
+
+    def result(self):
+        return self
+
 function_mappings = {
     GET_PROPERTY: GetProperty,
     GET_INPUT: GetInput,
-    GET_ATTRIBUTE: GetAttribute
+    GET_ATTRIBUTE: GetAttribute,
+    CONCAT: Concat
 }
 
 
