@@ -17,6 +17,7 @@ from toscaparser.common.exception import ExceptionCollector
 from toscaparser.common.exception import MissingRequiredFieldError
 from toscaparser.common.exception import UnknownFieldError
 from toscaparser.common.exception import ValidationError
+from toscaparser.elements.tosca_type_validation import TypeValidation
 from toscaparser.utils.gettextutils import _
 import toscaparser.utils.urlutils
 import toscaparser.utils.yamlparser
@@ -31,14 +32,18 @@ class ImportsLoader(object):
                       ('file', 'repository', 'namespace_uri',
                        'namespace_prefix')
 
-    def __init__(self, importslist, path, type_definition_list=None):
+    def __init__(self, importslist, path, type_definition_list=None,
+                 tpl=None):
         self.importslist = importslist
         self.custom_defs = {}
-        if not path:
+        if not path and not tpl:
             msg = _('Input tosca template is not provided.')
             log.warning(msg)
             ExceptionCollector.appendException(ValidationError(message=msg))
         self.path = path
+        self.repositories = {}
+        if tpl and tpl.get('repositories'):
+            self.repositories = tpl.get('repositories')
         self.type_definition_list = []
         if type_definition_list:
             if isinstance(type_definition_list, list):
@@ -73,14 +78,22 @@ class ImportsLoader(object):
 
                     custom_type = self._load_import_template(import_name,
                                                              import_uri)
-                    self._update_custom_def(custom_type)
+                    namespace_prefix = None
+                    if isinstance(import_uri, dict):
+                        namespace_prefix = import_uri.get(
+                            self.NAMESPACE_PREFIX)
+                    if custom_type:
+                        TypeValidation(custom_type, import_def)
+                        self._update_custom_def(custom_type, namespace_prefix)
             else:  # old style of imports
                 custom_type = self._load_import_template(None,
                                                          import_def)
                 if custom_type:
-                    self._update_custom_def(custom_type)
+                    TypeValidation(
+                        custom_type, import_def)
+                    self._update_custom_def(custom_type, None)
 
-    def _update_custom_def(self, custom_type):
+    def _update_custom_def(self, custom_type, namespace_prefix):
         outer_custom_types = {}
         for type_def in self.type_definition_list:
             outer_custom_types = custom_type.get(type_def)
@@ -88,7 +101,16 @@ class ImportsLoader(object):
                 if type_def == "imports":
                     self.custom_defs.update({'imports': outer_custom_types})
                 else:
-                    self.custom_defs.update(outer_custom_types)
+                    if namespace_prefix:
+                        prefix_custom_types = {}
+                        for type_def_key in outer_custom_types.keys():
+                            namespace_prefix_to_key = (namespace_prefix +
+                                                       "." + type_def_key)
+                            prefix_custom_types[namespace_prefix_to_key] = \
+                                outer_custom_types[type_def_key]
+                        self.custom_defs.update(prefix_custom_types)
+                    else:
+                        self.custom_defs.update(outer_custom_types)
 
     def _validate_import_keys(self, import_name, import_uri_def):
         if self.FILE not in import_uri_def.keys():
@@ -122,6 +144,8 @@ class ImportsLoader(object):
         +----------+--------+------------------------------+
         | file     | file   | OK                           |
         | file     | URL    | OK                           |
+        | preparsed| file   | file must be a full path     |
+        | preparsed| URL    | OK                           |
         | URL      | file   | file must be a relative path |
         | URL      | URL    | OK                           |
         +----------+--------+------------------------------+
@@ -132,13 +156,9 @@ class ImportsLoader(object):
             self._validate_import_keys(import_name, import_uri_def)
             file_name = import_uri_def.get(self.FILE)
             repository = import_uri_def.get(self.REPOSITORY)
-            namespace_uri = import_uri_def.get(self.NAMESPACE_URI)
-            # TODO(anyone) : will be extended this namespace_prefix in
-            # the next patch after design discussion with PTL.
-            # namespace_prefix = import_uri_def.get(self.NAMESPACE_PREFIX)
         else:
             file_name = import_uri_def
-            namespace_uri = None
+            repository = None
             short_import_notation = True
 
         if not file_name:
@@ -151,30 +171,62 @@ class ImportsLoader(object):
 
         if toscaparser.utils.urlutils.UrlUtils.validate_url(file_name):
             return YAML_LOADER(file_name, False)
-        elif not namespace_uri:
+        elif not repository:
             import_template = None
-            a_file = True
-            main_a_file = os.path.isfile(self.path)
-            if main_a_file:
-                if os.path.isfile(file_name):
+            if self.path:
+                if toscaparser.utils.urlutils.UrlUtils.validate_url(self.path):
+                    if os.path.isabs(file_name):
+                        msg = (_('Absolute file name "%(name)s" cannot be '
+                                 'used in a URL-based input template '
+                                 '"%(template)s".')
+                               % {'name': file_name, 'template': self.path})
+                        log.error(msg)
+                        ExceptionCollector.appendException(ImportError(msg))
+                        return
+                    import_template = toscaparser.utils.urlutils.UrlUtils.\
+                        join_url(self.path, file_name)
+                    a_file = False
+                else:
+                    a_file = True
+                    main_a_file = os.path.isfile(self.path)
+
+                    if main_a_file:
+                        if os.path.isfile(file_name):
+                            import_template = file_name
+                        else:
+                            full_path = os.path.join(
+                                os.path.dirname(os.path.abspath(self.path)),
+                                file_name)
+                            if os.path.isfile(full_path):
+                                import_template = full_path
+                            else:
+                                file_path = file_name.rpartition("/")
+                                dir_path = os.path.dirname(os.path.abspath(
+                                    self.path))
+                                if file_path[0] != '' and dir_path.endswith(
+                                    file_path[0]):
+                                        import_template = dir_path + "/" +\
+                                            file_path[2]
+                                        if not os.path.isfile(import_template):
+                                            msg = (_('"%(import_template)s" is'
+                                                     'not a valid file')
+                                                   % {'import_template':
+                                                      import_template})
+                                            log.error(msg)
+                                            ExceptionCollector.appendException
+                                            (ValueError(msg))
+            else:  # template is pre-parsed
+                if os.path.isabs(file_name) and os.path.isfile(file_name):
+                    a_file = True
                     import_template = file_name
                 else:
-                    full_path = os.path.join(
-                        os.path.dirname(os.path.abspath(self.path)),
-                        file_name)
-                    if os.path.isfile(full_path):
-                        import_template = full_path
-            else:  # main_a_url
-                if os.path.isabs(file_name):
-                    msg = (_('Absolute file name "%(name)s" cannot be used '
-                             'in a URL-based input template "%(template)s".')
-                           % {'name': file_name, 'template': self.path})
+                    msg = (_('Relative file name "%(name)s" cannot be used '
+                             'in a pre-parsed input template.')
+                           % {'name': file_name})
                     log.error(msg)
                     ExceptionCollector.appendException(ImportError(msg))
                     return
-                import_template = toscaparser.utils.urlutils.UrlUtils.\
-                    join_url(self.path, file_name)
-                a_file = False
+
             if not import_template:
                 log.error(_('Import "%(name)s" is not valid.') %
                           {'name': import_uri_def})
@@ -188,21 +240,31 @@ class ImportsLoader(object):
             log.error(_('Import "%(name)s" is not valid.') % import_uri_def)
             ExceptionCollector.appendException(
                 ImportError(_('Import "%s" is not valid.') % import_uri_def))
+            return
 
-        # Remove leading, ending spaces and strip the last character if "/"
-        namespace_uri = ((namespace_uri).strip()).rstrip("//")
+        full_url = ""
+        if repository:
+            if self.repositories:
+                for repo_name, repo_def in self.repositories.items():
+                    if repo_name == repository:
+                        # Remove leading, ending spaces and strip
+                        # the last character if "/"
+                        repo_url = ((repo_def['url']).strip()).rstrip("//")
+                        full_url = repo_url + "/" + file_name
 
-        if toscaparser.utils.urlutils.UrlUtils.validate_url(namespace_uri):
-            full_url = None
-            if repository:
-                repository = ((repository).strip()).rstrip("//")
-                full_url = namespace_uri + "/" + repository + "/" + file_name
-            else:
-                full_url = namespace_uri + "/" + file_name
+            if not full_url:
+                msg = (_('referenced repository "%(n_uri)s" in import '
+                         'definition "%(tpl)s" not found.')
+                       % {'n_uri': repository, 'tpl': import_name})
+                log.error(msg)
+                ExceptionCollector.appendException(ImportError(msg))
+                return
+
+        if toscaparser.utils.urlutils.UrlUtils.validate_url(full_url):
             return YAML_LOADER(full_url, False)
         else:
-            msg = (_('namespace_uri "%(n_uri)s" is not valid in import '
+            msg = (_('repository url "%(n_uri)s" is not valid in import '
                      'definition "%(tpl)s".')
-                   % {'n_uri': namespace_uri, 'tpl': import_name})
+                   % {'n_uri': repo_url, 'tpl': import_name})
             log.error(msg)
             ExceptionCollector.appendException(ImportError(msg))
