@@ -13,6 +13,7 @@
 from toscaparser.common import exception
 from toscaparser.elements.artifacttype import ArtifactTypeDef
 from toscaparser.elements.entity_type import EntityType
+from toscaparser.elements.grouptype import GroupType
 import toscaparser.elements.interfaces as ifaces
 from toscaparser.elements.nodetype import NodeType
 from toscaparser.elements.policytype import PolicyType
@@ -40,6 +41,7 @@ policy_placement_type = PolicyType('tosca.policies.Placement')
 policy_scaling_type = PolicyType('tosca.policies.Scaling')
 policy_update_type = PolicyType('tosca.policies.Update')
 policy_performance_type = PolicyType('tosca.policies.Performance')
+group_type = GroupType('tosca.groups.Root')
 
 
 class ToscaDefTest(TestCase):
@@ -56,40 +58,46 @@ class ToscaDefTest(TestCase):
         self.assertEqual(network_port_type.parent_type.type,
                          "tosca.nodes.Root")
 
+    def test_group(self):
+        self.assertEqual(group_type.type, "tosca.groups.Root")
+        self.assertIn(ifaces.LIFECYCLE_SHORTNAME, group_type.interfaces)
+
     def test_capabilities(self):
+        # Assure the normative Compute node type
+        # has all the required Capability types
+        # regardless of symbloc name
+        # TODO(Matt) - since Compute IS a normative node type
+        # we SHOULD test symbolic capability names as well
         self.assertEqual(
-            sorted(['tosca.capabilities.Container',
-                    'tosca.capabilities.OperatingSystem',
-                    'tosca.capabilities.network.Bindable',
-                    'tosca.capabilities.Scalable']),
+            ['tosca.capabilities.Container',
+             'tosca.capabilities.Node',
+             'tosca.capabilities.OperatingSystem',
+             'tosca.capabilities.Scalable',
+             'tosca.capabilities.network.Bindable'],
             sorted([c.type for c in compute_type.get_capabilities_objects()]))
+        # Assure the normative Network node type
+        # hsa all the required Capability types
+        # TODO(Matt) - since Network IS a normative node type
+        # we SHOULD test symbolic capability names as well
         self.assertEqual(
-            ['tosca.capabilities.network.Linkable'],
-            [c.type for c in network_type.get_capabilities_objects()])
-        endpoint_properties = ['initiator', 'network_name', 'port',
-                               'port_name', 'ports', 'protocol',
-                               'secure', 'url_path']
+            ['tosca.capabilities.Node',
+             'tosca.capabilities.network.Linkable'],
+            sorted([c.type for c in network_type.get_capabilities_objects()]))
+
+        # Assure the normative WebServer node type's
+        # Endpoint cap. has all required property names
+        # Note: we are testing them in alphabetic sort order
         endpoint_props_def_objects = \
             self._get_capability_properties_def_objects(
                 webserver_type.get_capabilities_objects(),
                 'tosca.capabilities.Endpoint')
+        # Assure WebServer's Endpoint capability's properties have their
+        # required keyname value set correctly
         self.assertEqual(
-            endpoint_properties,
-            sorted([p.name for p in endpoint_props_def_objects]))
-        for p in endpoint_props_def_objects:
-            if p.name in endpoint_properties:
-                self.assertFalse(p.required)
-        endpoint_props_def = self._get_capability_properties_def(
-            webserver_type.get_capabilities_objects(),
-            'tosca.capabilities.Endpoint')
-        self.assertEqual(
-            endpoint_properties,
-            sorted(endpoint_props_def.keys()))
-        endpoint_prop_def = self._get_capability_property_def(
-            webserver_type.get_capabilities_objects(),
-            'tosca.capabilities.Endpoint',
-            'initiator')
-        self.assertEqual(None, endpoint_prop_def)
+            [('initiator', False), ('network_name', False), ('port', False),
+             ('port_name', False), ('ports', False), ('protocol', True),
+             ('secure', False), ('url_path', False)],
+            sorted([(p.name, p.required) for p in endpoint_props_def_objects]))
 
         os_props = self._get_capability_properties_def_objects(
             compute_type.get_capabilities_objects(),
@@ -131,36 +139,35 @@ class ToscaDefTest(TestCase):
                 break
         return properties_def
 
-    def _get_capability_property_def(self, caps, type, property):
-        property_def = None
-        for cap in caps:
-            if cap.type == type:
-                property_def = cap.get_property_def_value(property)
-                break
-        return property_def
-
     def test_properties_def(self):
         self.assertEqual(
-            ['name', 'password', 'user'],
+            ['name', 'password', 'port', 'user'],
             sorted(database_type.get_properties_def().keys()))
 
     def test_attributes_def(self):
         self.assertEqual(
-            ['private_address', 'public_address'],
+            ['networks', 'ports', 'private_address', 'public_address',
+             'state', 'tosca_id', 'tosca_name'],
             sorted(compute_type.get_attributes_def().keys()))
 
     def test_requirements(self):
         self.assertEqual(
             [{'host': {'capability': 'tosca.capabilities.Container',
                        'node': 'tosca.nodes.Compute',
-                       'relationship': 'tosca.relationships.HostedOn'}}],
+                       'relationship': 'tosca.relationships.HostedOn'}},
+             {'dependency': {'capability': 'tosca.capabilities.Node',
+                             'node': 'tosca.nodes.Root',
+                             'occurrences': [0, 'UNBOUNDED'],
+                             'relationship': 'tosca.relationships.DependsOn'}}
+             ],
             [r for r in component_type.requirements])
 
     def test_relationship(self):
         self.assertEqual(
-            [('tosca.relationships.HostedOn', 'tosca.nodes.Compute')],
-            [(relation.type, node.type) for
-             relation, node in component_type.relationship.items()])
+            [('tosca.relationships.DependsOn', 'tosca.nodes.Root'),
+             ('tosca.relationships.HostedOn', 'tosca.nodes.Compute')],
+            sorted([(relation.type, node.type) for
+                   relation, node in component_type.relationship.items()]))
         self.assertIn(
             ('tosca.relationships.HostedOn', ['tosca.capabilities.Container']),
             [(relation.type, relation.valid_target_types) for
