@@ -16,6 +16,7 @@ import six
 from toscaparser.common import exception
 import toscaparser.elements.interfaces as ifaces
 from toscaparser.elements.nodetype import NodeType
+from toscaparser.elements.portspectype import PortSpec
 from toscaparser.functions import GetInput
 from toscaparser.functions import GetProperty
 from toscaparser.nodetemplate import NodeTemplate
@@ -67,7 +68,7 @@ class ToscaTemplateTest(TestCase):
         tpl_name = "mysql_database"
         expected_type = "tosca.nodes.Database"
         expected_properties = ['name', 'password', 'user']
-        expected_capabilities = ['database_endpoint']
+        expected_capabilities = ['database_endpoint', 'feature']
         expected_requirements = [{'host': 'mysql_dbms'}]
         ''' TODO: needs enhancement in tosca_elk.yaml..
         expected_relationshp = ['tosca.relationships.HostedOn']
@@ -348,6 +349,11 @@ class ToscaTemplateTest(TestCase):
             NotImplementedError,
             lambda: NodeTemplate(tpl_name, nodetemplates).relationships)
 
+    # Test the following:
+    # 1. Custom node type derived from 'WebApplication' named 'TestApp'
+    #    with a custom Capability Type 'TestCapability'
+    # 2. Same as #1, but referencing a custom 'TestCapability' Capability Type
+    #    that is not defined
     def test_custom_capability_type_definition(self):
         tpl_snippet = '''
         node_templates:
@@ -358,7 +364,7 @@ class ToscaTemplateTest(TestCase):
                 properties:
                   test: 1
         '''
-        # custom definition with capability type definition
+        # custom node type definition with custom capability type definition
         custom_def = '''
         tosca.nodes.WebApplication.TestApp:
           derived_from: tosca.nodes.WebApplication
@@ -372,7 +378,7 @@ class ToscaTemplateTest(TestCase):
               type: integer
               required: false
         '''
-        expected_capabilities = ['test_cap']
+        expected_capabilities = ['app_endpoint', 'feature', 'test_cap']
         nodetemplates = (toscaparser.utils.yamlparser.
                          simple_parse(tpl_snippet))['node_templates']
         custom_def = (toscaparser.utils.yamlparser.
@@ -383,7 +389,7 @@ class ToscaTemplateTest(TestCase):
             expected_capabilities,
             sorted(tpl.get_capabilities().keys()))
 
-        # custom definition without capability type definition
+        # custom definition without valid capability type definition
         custom_def = '''
         tosca.nodes.WebApplication.TestApp:
           derived_from: tosca.nodes.WebApplication
@@ -704,3 +710,61 @@ class ToscaTemplateTest(TestCase):
             os.path.dirname(os.path.abspath(__file__)),
             "data/test_attributes_inheritance.yaml")
         ToscaTemplate(tosca_tpl)
+
+    def test_repositories_definition(self):
+        tosca_tpl = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)),
+            "data/test_repositories_definition.yaml")
+        ToscaTemplate(tosca_tpl)
+
+    def test_custom_caps_def(self):
+        tosca_tpl = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)),
+            "data/test_custom_caps_def.yaml")
+        ToscaTemplate(tosca_tpl)
+
+    def test_custom_rel_with_script(self):
+        tosca_tpl = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)),
+            "data/test_tosca_custom_rel_with_script.yaml")
+        tosca = ToscaTemplate(tosca_tpl)
+        rel = tosca.relationship_templates[0]
+        self.assertEqual(len(rel.interfaces), 1)
+        self.assertEqual(rel.interfaces[0].type, "Configure")
+
+    def test_various_portspec_errors(self):
+        tosca_tpl = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)),
+            "data/datatypes/test_datatype_portspec_add_req.yaml")
+        self.assertRaises(exception.ValidationError, ToscaTemplate, tosca_tpl,
+                          None)
+
+        # TODO(TBD) find way to reuse error messages from constraints.py
+        msg = (_('The value "%(pvalue)s" of property "%(pname)s" is out of '
+                 'range "(min:%(vmin)s, max:%(vmax)s)".') %
+               dict(pname=PortSpec.SOURCE,
+                    pvalue='0',
+                    vmin='1',
+                    vmax='65535'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.ValidationError, msg)
+
+        # Test value below range min.
+        msg = (_('The value "%(pvalue)s" of property "%(pname)s" is out of '
+                 'range "(min:%(vmin)s, max:%(vmax)s)".') %
+               dict(pname=PortSpec.SOURCE,
+                    pvalue='1',
+                    vmin='2',
+                    vmax='65534'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.RangeValueError, msg)
+
+        # Test value above range max.
+        msg = (_('The value "%(pvalue)s" of property "%(pname)s" is out of '
+                 'range "(min:%(vmin)s, max:%(vmax)s)".') %
+               dict(pname=PortSpec.SOURCE,
+                    pvalue='65535',
+                    vmin='2',
+                    vmax='65534'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.RangeValueError, msg)
