@@ -18,9 +18,12 @@ from toscaparser.imports import ImportsLoader
 from toscaparser.nodetemplate import NodeTemplate
 from toscaparser.parameters import Input
 from toscaparser.parameters import Output
+from toscaparser.policy import Policy
 from toscaparser.relationship_template import RelationshipTemplate
 from toscaparser.tests.base import TestCase
+from toscaparser.topology_template import TopologyTemplate
 from toscaparser.tosca_template import ToscaTemplate
+from toscaparser.triggers import Triggers
 from toscaparser.utils.gettextutils import _
 
 import toscaparser.utils.yamlparser
@@ -53,6 +56,48 @@ class ToscaTemplateValidationTest(TestCase):
             _('Template contains unknown field "node_template". Refer to the '
               'definition to verify valid values.'))
 
+    def test_template_with_imports_validation(self):
+        tpl_path = os.path.join(
+            os.path.dirname(os.path.abspath(__file__)),
+            "data/tosca_imports_validation.yaml")
+        self.assertRaises(exception.ValidationError, ToscaTemplate, tpl_path)
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.UnknownFieldError,
+            _('Template custom_types/imported_sample.yaml contains unknown '
+              'field "descriptions". Refer to the definition'
+              ' to verify valid values.'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.UnknownFieldError,
+            _('Template custom_types/imported_sample.yaml contains unknown '
+              'field "node_typess". Refer to the definition to '
+              'verify valid values.'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.UnknownFieldError,
+            _('Template custom_types/imported_sample.yaml contains unknown '
+              'field "tosca1_definitions_version". Refer to the definition'
+              ' to verify valid values.'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.InvalidTemplateVersion,
+            _('The template version "tosca_simple_yaml_1_10 in '
+              'custom_types/imported_sample.yaml" is invalid. '
+              'Valid versions are "tosca_simple_yaml_1_0, '
+              'tosca_simple_profile_for_nfv_1_0_0".'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.UnknownFieldError,
+            _('Template custom_types/imported_sample.yaml contains unknown '
+              'field "policy_types1". Refer to the definition to '
+              'verify valid values.'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.UnknownFieldError,
+            _('Nodetype"tosca.nodes.SoftwareComponent.Logstash" contains '
+              'unknown field "capabilities1". Refer to the definition '
+              'to verify valid values.'))
+        exception.ExceptionCollector.assertExceptionMessage(
+            exception.UnknownFieldError,
+            _('Policy "mycompany.mytypes.myScalingPolicy" contains unknown '
+              'field "derived1_from". Refer to the definition to '
+              'verify valid values.'))
+
     def test_inputs(self):
         tpl_snippet = '''
         inputs:
@@ -123,17 +168,29 @@ tosca-parser/master/toscaparser/tests/data/custom_types/wordpress.yaml
         imports:
           - some_definitions: custom_types/paypalpizzastore_nodejs_app.yaml
           - more_definitions:
-              file: toscaparser/tests/data/custom_types/wordpress.yaml
-              repository: tosca-parser/master
-              namespace_uri: https://raw.githubusercontent.com/openstack
+              file: 'https://raw.githubusercontent.com/openstack/tosca-parser\
+/master/toscaparser/tests/data/custom_types/wordpress.yaml'
               namespace_prefix: single_instance_wordpress
         '''
         path = 'toscaparser/tests/data/tosca_elk.yaml'
         custom_defs = self._imports_content_test(tpl_snippet,
                                                  path,
                                                  "node_types")
-        self.assertTrue(custom_defs.get("tosca.nodes."
-                                        "WebApplication.WordPress"))
+        self.assertTrue(custom_defs.get("single_instance_wordpress.tosca."
+                                        "nodes.WebApplication.WordPress"))
+
+    def test_imports_wth_namespace_prefix(self):
+        tpl_snippet = '''
+        imports:
+          - more_definitions:
+              file: custom_types/nested_rsyslog.yaml
+              namespace_prefix: testprefix
+        '''
+        path = 'toscaparser/tests/data/tosca_elk.yaml'
+        custom_defs = self._imports_content_test(tpl_snippet,
+                                                 path,
+                                                 "node_types")
+        self.assertTrue(custom_defs.get("testprefix.Rsyslog"))
 
     def test_imports_with_no_main_template(self):
         tpl_snippet = '''
@@ -187,7 +244,6 @@ tosca-parser/master/toscaparser/tests/data/custom_types/wordpress.yaml
           - more_definitions:
               file: https://raw.githubusercontent.com/openstack/\
 tosca-parser/master/toscaparser/tests/data/custom_types/wordpress.yaml
-              namespace_prefix: mycompany
         '''
         path = 'https://raw.githubusercontent.com/openstack/\
 tosca-parser/master/toscaparser/tests/data/\
@@ -202,33 +258,31 @@ tosca_single_instance_wordpress_with_url_import.yaml'
         tpl_snippet = '''
         imports:
           - more_definitions:
-              file: heat-translator/master/translator/tests/data/\
-custom_types/wordpress.yaml
-              namespace_uri: https://raw.githubusercontent.com/openstack/
-              namespace_prefix: mycompany
+             file: https://raw.githubusercontent.com/openstack/\
+heat-translator/master/translator/tests/data/custom_types/wordpress.yaml
+             namespace_prefix: mycompany
+             namespace_uri: http://docs.oasis-open.org/tosca/ns/simple/yaml/1.0
         '''
         path = 'toscaparser/tests/data/tosca_elk.yaml'
         custom_defs = self._imports_content_test(tpl_snippet,
                                                  path,
                                                  "node_types")
-        self.assertTrue(custom_defs.get("tosca.nodes."
+        self.assertTrue(custom_defs.get("mycompany.tosca.nodes."
                                         "WebApplication.WordPress"))
 
-    def test_import_error_namespace_uri(self):
+    def test_import_error_file_uri(self):
         tpl_snippet = '''
         imports:
           - more_definitions:
-              file: toscaparser/tests/data/tosca_elk.yaml
-              namespace_uri: mycompany.com/ns/tosca/2.0
-              namespace_prefix: mycompany
+             file: mycompany.com/ns/tosca/2.0/toscaparser/tests/data\
+/tosca_elk.yaml
+             namespace_prefix: mycompany
+             namespace_uri: http://docs.oasis-open.org/tosca/ns/simple/yaml/1.0
         '''
-        errormsg = _('namespace_uri "mycompany.com/ns/tosca/2.0" is not '
-                     'valid in import definition "more_definitions".')
         path = 'toscaparser/tests/data/tosca_elk.yaml'
-        err = self.assertRaises(ImportError,
-                                self._imports_content_test,
-                                tpl_snippet, path, None)
-        self.assertEqual(errormsg, err.__str__())
+        self.assertRaises(ImportError,
+                          self._imports_content_test,
+                          tpl_snippet, path, None)
 
     def test_import_single_line_error(self):
         tpl_snippet = '''
@@ -281,6 +335,137 @@ custom_types/wordpress.yaml
                                'to verify valid values.'),
                              err.__str__())
 
+    def test_groups(self):
+        tpl_snippet = '''
+        node_templates:
+          server:
+            type: tosca.nodes.Compute
+            requirements:
+              - log_endpoint:
+                  capability: log_endpoint
+
+          mysql_dbms:
+            type: tosca.nodes.DBMS
+            properties:
+              root_password: aaa
+              port: 3376
+
+        groups:
+          webserver_group:
+            type: tosca.groups.Root
+            members: [ server, mysql_dbms ]
+        '''
+        tpl = (toscaparser.utils.yamlparser.simple_parse(tpl_snippet))
+        TopologyTemplate(tpl, None)
+
+    def test_groups_with_missing_required_field(self):
+        tpl_snippet = '''
+        node_templates:
+          server:
+            type: tosca.nodes.Compute
+            requirements:
+              - log_endpoint:
+                  capability: log_endpoint
+
+          mysql_dbms:
+            type: tosca.nodes.DBMS
+            properties:
+              root_password: aaa
+              port: 3376
+
+        groups:
+          webserver_group:
+              members: ['server', 'mysql_dbms']
+        '''
+        tpl = (toscaparser.utils.yamlparser.simple_parse(tpl_snippet))
+        err = self.assertRaises(exception.MissingRequiredFieldError,
+                                TopologyTemplate, tpl, None)
+        expectedmessage = _('Template "webserver_group" is missing '
+                            'required field "type".')
+        self.assertEqual(expectedmessage, err.__str__())
+
+    def test_groups_with_unknown_target(self):
+        tpl_snippet = '''
+        node_templates:
+          server:
+            type: tosca.nodes.Compute
+            requirements:
+              - log_endpoint:
+                  capability: log_endpoint
+
+          mysql_dbms:
+            type: tosca.nodes.DBMS
+            properties:
+              root_password: aaa
+              port: 3376
+
+        groups:
+          webserver_group:
+            type: tosca.groups.Root
+            members: [ serv, mysql_dbms ]
+        '''
+        tpl = (toscaparser.utils.yamlparser.simple_parse(tpl_snippet))
+        expectedmessage = _('"Target member "serv" is not found in '
+                            'node_templates"')
+        err = self.assertRaises(exception.InvalidGroupTargetException,
+                                TopologyTemplate, tpl, None)
+        self.assertEqual(expectedmessage, err.__str__())
+
+    def test_groups_with_repeated_targets(self):
+        tpl_snippet = '''
+        node_templates:
+          server:
+            type: tosca.nodes.Compute
+            requirements:
+              - log_endpoint:
+                  capability: log_endpoint
+
+          mysql_dbms:
+            type: tosca.nodes.DBMS
+            properties:
+              root_password: aaa
+              port: 3376
+
+        groups:
+          webserver_group:
+            type: tosca.groups.Root
+            members: [ server, server, mysql_dbms ]
+        '''
+        tpl = (toscaparser.utils.yamlparser.simple_parse(tpl_snippet))
+        expectedmessage = _('"Member nodes '
+                            '"[\'server\', \'server\', \'mysql_dbms\']" '
+                            'should be >= 1 and not repeated"')
+        err = self.assertRaises(exception.InvalidGroupTargetException,
+                                TopologyTemplate, tpl, None)
+        self.assertEqual(expectedmessage, err.__str__())
+
+    def test_groups_with_only_one_target(self):
+        tpl_snippet = '''
+        node_templates:
+          server:
+            type: tosca.nodes.Compute
+            requirements:
+              - log_endpoint:
+                  capability: log_endpoint
+
+          mysql_dbms:
+            type: tosca.nodes.DBMS
+            properties:
+              root_password: aaa
+              port: 3376
+
+        groups:
+          webserver_group:
+            type: tosca.groups.Root
+            members: []
+        '''
+        tpl = (toscaparser.utils.yamlparser.simple_parse(tpl_snippet))
+        expectedmessage = _('"Member nodes "[]" should be >= 1 '
+                            'and not repeated"')
+        err = self.assertRaises(exception.InvalidGroupTargetException,
+                                TopologyTemplate, tpl, None)
+        self.assertEqual(expectedmessage, err.__str__())
+
     def _custom_types(self):
         custom_types = {}
         def_file = os.path.join(
@@ -962,34 +1147,6 @@ custom_types/wordpress.yaml
                'are "%s".') % valid_versions))
 
     def test_node_template_capabilities_properties(self):
-        tpl_snippet = '''
-        node_templates:
-          server:
-            type: tosca.nodes.Compute
-            capabilities:
-              host:
-                properties:
-                  disk_size: 10 GB
-                  num_cpus: { get_input: cpus }
-                  mem_size: 4096 MB
-              os:
-                properties:
-                  architecture: x86_64
-                  type: Linux
-                  distribution: Fedora
-                  version: 18.0
-              scalable:
-                properties:
-                  min_instances: 1
-                  default_instances: 5
-        '''
-        expectedmessage = _('"properties" of template "server" is missing '
-                            'required field "[\'max_instances\']".')
-        err = self.assertRaises(
-            exception.MissingRequiredFieldError,
-            lambda: self._single_node_template_content_test(tpl_snippet))
-        self.assertEqual(expectedmessage, err.__str__())
-
         # validating capability property values
         tpl_snippet = '''
         node_templates:
@@ -1107,3 +1264,118 @@ custom_types/wordpress.yaml
             metadata: none
         '''
         self._single_node_template_content_test(tpl_snippet_metadata_inline)
+
+    def test_policy_valid_keynames(self):
+        tpl_snippet = '''
+        policies:
+          - servers_placement:
+              type: tosca.policies.Placement
+              description: Apply placement policy to servers
+              metadata: { user1: 1001, user2: 1002 }
+              targets: [ serv1, serv2 ]
+        '''
+        policies = (toscaparser.utils.yamlparser.
+                    simple_parse(tpl_snippet))['policies'][0]
+        name = list(policies.keys())[0]
+        Policy(name, policies[name], None, None)
+
+    def test_policy_invalid_keyname(self):
+        tpl_snippet = '''
+        policies:
+          - servers_placement:
+              type: tosca.policies.Placement
+              testkey: testvalue
+        '''
+        policies = (toscaparser.utils.yamlparser.
+                    simple_parse(tpl_snippet))['policies'][0]
+        name = list(policies.keys())[0]
+
+        expectedmessage = _('Policy "servers_placement" contains '
+                            'unknown field "testkey". Refer to the '
+                            'definition to verify valid values.')
+        err = self.assertRaises(
+            exception.UnknownFieldError,
+            lambda: Policy(name, policies[name], None, None))
+        self.assertEqual(expectedmessage, err.__str__())
+
+    def test_policy_trigger_valid_keyname(self):
+        tpl_snippet = '''
+        triggers:
+         - resize_compute:
+             description: trigger
+             event_type: tosca.events.resource.utilization
+             schedule:
+               start_time: "2015-05-07T07:00:00Z"
+               end_time: "2015-06-07T07:00:00Z"
+             target_filter:
+               node: master-container
+               requirement: host
+               capability: Container
+             condition:
+               constraint: utilization greater_than 50%
+               period: 60
+               evaluations: 1
+               method : average
+             action:
+               resize: # Operation name
+                inputs:
+                 strategy: LEAST_USED
+                 implementation: Senlin.webhook()
+        '''
+        triggers = (toscaparser.utils.yamlparser.
+                    simple_parse(tpl_snippet))['triggers'][0]
+        name = list(triggers.keys())[0]
+        Triggers(name, triggers[name])
+
+    def test_policy_trigger_invalid_keyname(self):
+        tpl_snippet = '''
+        triggers:
+         - resize_compute:
+             description: trigger
+             event_type: tosca.events.resource.utilization
+             schedule:
+               start_time: "2015-05-07T07:00:00Z"
+               end_time: "2015-06-07T07:00:00Z"
+             target_filter1:
+               node: master-container
+               requirement: host
+               capability: Container
+             condition:
+               constraint: utilization greater_than 50%
+               period1: 60
+               evaluations: 1
+               method: average
+             action:
+               resize: # Operation name
+                inputs:
+                 strategy: LEAST_USED
+                 implementation: Senlin.webhook()
+        '''
+        triggers = (toscaparser.utils.yamlparser.
+                    simple_parse(tpl_snippet))['triggers'][0]
+        name = list(triggers.keys())[0]
+        expectedmessage = _(
+            'Triggers "resize_compute" contains unknown field '
+            '"target_filter1". Refer to the definition '
+            'to verify valid values.')
+        err = self.assertRaises(
+            exception.UnknownFieldError,
+            lambda: Triggers(name, triggers[name]))
+        self.assertEqual(expectedmessage, err.__str__())
+
+    def test_policy_missing_required_keyname(self):
+        tpl_snippet = '''
+        policies:
+          - servers_placement:
+              description: test description
+        '''
+        policies = (toscaparser.utils.yamlparser.
+                    simple_parse(tpl_snippet))['policies'][0]
+        name = list(policies.keys())[0]
+
+        expectedmessage = _('Template "servers_placement" is missing '
+                            'required field "type".')
+        err = self.assertRaises(
+            exception.MissingRequiredFieldError,
+            lambda: Policy(name, policies[name], None, None))
+        self.assertEqual(expectedmessage, err.__str__())
